#!/usr/bin/perl -w
#
# nl2csv.pl - v0.1 
# Translation of a particular FTN St. Louis Format Nodelist file
# into a comma delimited (*.csv) textmode database file.   
# Copyright (c) 2001-2002 Robert James Clay.  All Rights Reserved.
# This is free software;  you can redistribute it and/or
# modify it under the same terms as Perl itself.

use strict;
use Getopt::Std;
use vars qw/ $opt_n $opt_f $opt_e $opt_l $opt_h $opt_o $opt_v $opt_x /;

use FTN::Utility qw(&logging);

getopts('n:f:l:o:ehvx');

if ($opt_h) {
    &help ();	#printing usage/help message
    exit 0;
}

my ($nldir, $nlfile, $csvfile, $type, $num, $name, $loc, $nlline,
    $nametmp, $loctmp, $Logfile, $sysop, $phone, $bps, $flags, $tblname );
 
if ($opt_l) {
    $Logfile=$opt_l;  # this needs to be the filename & path
    undef $opt_l;
} else {
    $Logfile="nl2csv.log";	# default log file is in current dir
}

&log("Starting... ");

# note that "opt_v" is the verbose variable
if ($opt_v) {
    &log("Verbose flag is set");
}

# note that "opt_x" is the debug variable
if ($opt_x) {
    &log("Debug flag is set");
}

#  setup nodelist file variables
if ($opt_n) {

    $nldir = $opt_n;   # set nodelist directory variable
    
    if ($opt_f) {
        if ($opt_e) { # if set, then -f option must be exact file name
	    &log("Using exact file name &opt_f.");
	    $nlfile=$opt_f;
	} else { # if not set, then -f option is basename of nodelist file
	    $nlfile=&getnlfilename($opt_f);
	}

    } else { # if not set, 
        #  filename defaults to basename nodelist
        $nlfile=&getnlfilename("nodelist");
    }

    if ($opt_o) {

        $csvfile=$opt_o;
	
    } else { # if not set, 
        #  filename defaults to nodelist.csv
        $csvfile="nodelist.csv";
    }

} else {
    print "\nThe Nodelist directory variable must be set... \n";
    &help();
    exit (1);	#  exit after displaying usage if not set
}


&log("Nodelist directory: '$nldir'");
&log("Nodelist file: '$nlfile'");
&log("CSV file: '$csvfile'");

if ($opt_x) {
    print "Nodelist file is '$nldir.$nlfile' ..\n";
}

open (NODELIST, "$nldir/$nlfile")
    or die &log("Cannot open input file $nldir/$nlfile");

open (CSVFILE, ">$nldir/$csvfile")
    or die &log("Cannot open output file $nldir/$csvfile");

if ($opt_x) {
    &log("Debug mode is set");
}

#  set defaults
my $zone = 1;
my $net  = 0;
my $node = 0;
my $point = 0;
my $region = 0;

if ($opt_v) {	
    &log("Using nodelist $nlfile.");
}

while(<NODELIST>) {
    if ( /^;/ || /^\cZ/ ) {
#	print;
	next;
    }

    ($type,$num,$nlline) = split(',', $_, 3);
#    ($type,$num,$name,$loc,$sysop,$phone,$bps,$flags) = split(',', $_, 8);


    if($type eq "Zone") {	# Zone line
	$zone = $num;
	$net  = $num;
	$node = 0;
    }				# 
    elsif($type eq "Region") {	# Region line
	$region  = $num;
 	$net  = $num;
	$node = 0;
    }
    elsif($type eq "Host") {	# Host line
	$net = $num;
	$node = 0;
    }
    else {
	$node = $num;
    }

# print nodelist line to nodelist.csv file
    print CSVFILE "$zone:$net/$node";
    print CSVFILE ",$type,";
    print CSVFILE "$nlline";

}


close NODELIST;

close CSVFILE;

&log("Ending... ");

exit();

############################################
# subroutines
############################################

# Help message
############################################
sub help {
    print "\nUsage: nl2sql.pl -n nldir [-f nlfile] [-l logfile] [-o outfile] [-e] [-v] [-x]...\n";
    print "    nldir = nodelist directory...\n";
    print "    nlfile= nodelist filename, defaults to 'nodelist' in nldir. \n";
    print "    [-o outfile] = output filname (defaults to nodelist.csv in current dir)\n";
    print "    [-l logfile] = log filename (defaults to nl2csv.log in current dir)\n";
    print "   -e   If present, then nlfile & outfile are exact filenames.\n";
    print "   -v   Verbose Mode\n";
    print "   -x   Debug Mode\n";
}

################################################
# get nodelist filename, given path & base name
################################################
sub getnlfilename {
# Find the most recent version (by day number) when given a base name & dir
# of the nodelist;  once this is implemented, this will be the default. 
# Note that if there is more than one file with the same base name, it will
# use the first one found.

    my ($i, @files);

    my($basename) = @_;

    if ($opt_v) {&log("Searching for $basename files.")};

    opendir(DIR, $nldir);
	@files = grep(/$basename\.[0-9][0-9][0-9]$/i, readdir(DIR));
    closedir(DIR);

    if ($#files == -1) {
	&log("Nodelist files $basename not found");
	print ("\nNodelist files $basename not found.\n");
	&help;	
	exit();
    } else {
        if ($opt_v) {
	    for ($i=0; $i<@files; $i++) {
		&log("Nodelist file $i found: $files[$i]");
	    }
        }
    }

    if ($#files > 1) {
	&log("More than one '$basename' found, using first.");
    }
    
    return ($files[0]);   # return filename

}


#############################################
#  log subrouting.  requires FTN::Utility
#############################################
sub log {
#
    my(@text) = @_;
    my $progid="nl2csv";

    &logging($Logfile, $progid, @text);

}
